"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rs = require("recursive-readdir-async");
const fs = require("fs-extra");
const upath_1 = require("upath");
const WorkspaceConfig_1 = require("./WorkspaceConfig");
const WorkspaceTheme_1 = require("./WorkspaceTheme");
const Config_1 = require("./Config");
const File_1 = require("./File");
class Workspace {
    constructor(name) {
        this.files = [];
        this.name = name;
        this.path = Workspace.getDirectoryPath(name);
        this.config = new WorkspaceConfig_1.default(this.path, 'cli.conf.yaml');
        this.portalConfig = new Config_1.default(this.path, 'portal.conf.yaml');
        this.routerConfig = new Config_1.default(this.path, 'router.conf.yaml');
    }
    static async init(name, disableSSLVerification, ignoreSpecs) {
        if ((await this.exists(name)) === false) {
            throw new Error();
        }
        const workspace = new Workspace(name);
        await workspace.config.load();
        await workspace.portalConfig.load();
        await workspace.routerConfig.load();
        if (process.env.KONG_ADMIN_URL) {
            workspace.config.data.kong_admin_url = process.env.KONG_ADMIN_URL;
        }
        if (process.env.KONG_ADMIN_TOKEN) {
            workspace.config.data.kong_admin_token = process.env.KONG_ADMIN_TOKEN;
        }
        if (disableSSLVerification) {
            workspace.config.data.disable_ssl_verification = true;
        }
        if (ignoreSpecs) {
            workspace.config.data.ignore_specs = true;
        }
        return workspace;
    }
    async scan() {
        let files = await rs.list(this.path, { exclude: ['.DS_Store', 'cli.conf.yaml'] });
        if (files) {
            this.files = files.map((file) => {
                return new File_1.default(file.fullname, this.path);
            });
        }
    }
    static async exists(name) {
        try {
            const stat = await fs.lstat(Workspace.getDirectoryPath(name));
            return stat && stat.isDirectory();
        }
        catch (e) {
            return false;
        }
    }
    static getDirectoryPath(name) {
        return upath_1.join(process.cwd(), 'workspaces', name);
    }
    async getThemes() {
        let workspaceThemes = [];
        let themes = await rs.list(upath_1.join(this.path, 'themes'), {
            recursive: false,
            ignoreFolders: false,
        });
        themes = themes.filter((element) => element.isDirectory);
        for (let theme of themes) {
            workspaceThemes.push(await this.getTheme(theme.name));
        }
        return workspaceThemes;
    }
    getCurrentThemeName() {
        return this.portalConfig.data.theme.name;
    }
    async getTheme(name) {
        return await WorkspaceTheme_1.default.init(this.path, name);
    }
    async getCurrentTheme() {
        return this.getTheme(this.getCurrentThemeName());
    }
}
exports.default = Workspace;
